#ifndef SENSORS_H
#define SENSORS_H

#define SENSOR_CURRENT_API_VERSION	0x00010000	// 16 hi bits major, 16 low bits minor

#ifdef __BEOS__
	#include <SupportDefs.h>
	#include <drivers/Drivers.h>

	// Sensor-kind drivers base for op-codes :
	#define SENSOR_DRIVER_OP_CODES_BASE		B_DEVICE_OP_CODES_END
	#define sensor_time_t					bigtime_t
#else
	#include <time.h>

	#define sensor_time time_t	
#endif


#define SENSOR_MAX_NAME_LENGTH	(48)

typedef enum {
	INVALID_SENSOR_KIND		= -1,
	SENSOR_KIND_TEMPERATURE = 0,
	SENSOR_KIND_FAN,
	SENSOR_KIND_VOLTAGE,
	SENSOR_KIND_STATE,	// Case Opened, etc
	SENSOR_KIND_OTHER
} SENSOR_KIND;

typedef enum {
	SENSOR_UNIT_CLASS_VOLT,
	SENSOR_UNIT_CLASS_AMPERE,
	SENSOR_UNIT_CLASS_CELSIUS,
	SENSOR_UNIT_CLASS_HERTZ,
	SENSOR_UNIT_CLASS_OTHER
} SENSOR_UNIT_CLASS_ID;

typedef struct {
	SENSOR_UNIT_CLASS_ID	id;
	char					name [SENSOR_MAX_NAME_LENGTH + 1];
} SENSOR_UNIT_CLASS;

typedef struct {
	SENSOR_UNIT_CLASS_ID	class_id;
	float					class_unit_multiplier;
	char					name [SENSOR_MAX_NAME_LENGTH + 1];
} SENSOR_UNIT;


typedef struct _SENSOR	{
	int				id;
	char			name [SENSOR_MAX_NAME_LENGTH + 1];
	SENSOR_KIND		kind;	
	long			flags;
	
#define SENSOR_FLAG_CONTROLABLE		(0x01)
#define SENSOR_FLAG_ALARMABLE		(0x02)
#define SENSOR_FLAG_LOW_LIMIT		(0x04)
#define SENSOR_FLAG_HIGH_LIMIT		(0x08)
#define SENSOR_FLAG_LIMITS 			(SENSOR_FLAG_LOW_LIMIT | SENSOR_FLAG_HIGH_LIMIT)

	SENSOR_UNIT		unit;

	// float			min_value;			// Min Sensor Value
	// float			max_value;			// Max Sensor Value
	// Currents values
	sensor_time_t	when;					// At what time?
	
	// Sensor current value : value_multiplier * value / value_divider
	int				value;					// Current value
	int				value_multiplier;		// Current value multiplier
	int				value_divider;			// Current value divider, 0 = sensor disabled
	
	// float			alarm_low_limit;	// Triger alarm if under this low limit
	// float			alarm_high_limit;	// Triger alarm if over this high limit
} SENSOR;


typedef struct _SENSORS_MONITOR {
	char 			name[SENSOR_MAX_NAME_LENGTH + 1];
	int32			api_version;
	int 			nb_sensors;
} SENSORS_MONITOR;


// ioctl op-codes
enum {
	SENSOR_GET_MONITOR_INFO		=  SENSOR_DRIVER_OP_CODES_BASE,
	SENSOR_GET_VALUE,
	SENSOR_COMMAND
};
	
// Format standard d'un read() sur un /dev/misc/sensor/xxxxx :
// <sensor_id>\t[<sensor_name>]\t<sensor_value>\n


#endif // SENSORS_H