#ifndef W8378XX_H
#define W8378XX_H

// References
// ----------
// 
// "W93781D Winbond H/W Monitoring IC Data Sheet", w83781d.pdf, 13/1/1998, revision 0.63
// "W83782D Winbond H/W Monitoring IC Data Sheet", w83782d.pdf, ?/4/1998, revision 0.55

#include "Sensors.h"

#ifdef DRIVER_NAME
	#undef DRIVER_NAME
#endif
#define DRIVER_NAME 			"w8378xx"

#define W8378xx_UPDATE_PERIOD	(1500000)	// every second and half, expressed in microseconds


// Okay, the IO port :
// -------------------

#define W8378xx_IO_BASE		(0x290)	// base of IO ports

// Index Register port
#define W8378xx_ADDRESS		(0x5)	
#define W8378xx_IO_ADDRESS	(W8378xx_IO_BASE + W8378xx_ADDRESS)	
#define W8378xx_BUSY		(0x80)	// flag set in address port when chip is busy
#define	W8378xx_BUSY_WAIT	(1000)	// 1000 us
	
// Data Register port		
#define W8378xx_DATA		(0x6)	
#define W8378xx_IO_DATA		(W8378xx_IO_BASE + W8378xx_DATA)	

// Registers indexes list :
// ------------------------

// W8378xx_VALUE_<name> >>> 'name' Value Register
// W8378xx_REG_<name>	>>> 'name' Register
// W8378xx_<name>_????	>>> masks, flags, constantes for 'name' Register

// ---- Value RAM *registers* (p25) Bank 0
#define	W8378xx_VALUE_RAM_AUTOINDEX	(0x060)
#define	W8378xx_VALUE_RAM 			(0x020)
enum {
	W8378xx_VALUE_IN0 = W8378xx_VALUE_RAM,
	W8378xx_VALUE_IN1,
	W8378xx_VALUE_IN2,
	W8378xx_VALUE_IN3,
	W8378xx_VALUE_IN4,
	W8378xx_VALUE_MINUS_IN5,
	W8378xx_VALUE_MINUS_IN6,
	W8378xx_VALUE_TEMPERATURE1,
	W8378xx_VALUE_FAN1,
	W8378xx_VALUE_FAN2,
	W8378xx_VALUE_FAN3,
	W8378xx_VALUE_IN0_HIGH_LIMIT,
	W8378xx_VALUE_IN0_LOW_LIMIT,
	W8378xx_VALUE_IN1_HIGH_LIMIT,
	W8378xx_VALUE_IN1_LOW_LIMIT,
	W8378xx_VALUE_IN2_HIGH_LIMIT,
	W8378xx_VALUE_IN2_LOW_LIMIT,
	W8378xx_VALUE_IN3_HIGH_LIMIT,
	W8378xx_VALUE_IN3_LOW_LIMIT,
	W8378xx_VALUE_IN4_HIGH_LIMIT,
	W8378xx_VALUE_IN4_LOW_LIMIT,
	W8378xx_VALUE_MINUS_IN5_HIGH_LIMIT,
	W8378xx_VALUE_MINUS_IN5_LOW_LIMIT,
	W8378xx_VALUE_MINUS_IN6_HIGH_LIMIT,
	W8378xx_VALUE_MINUS_IN6_LOW_LIMIT,
	W8378xx_VALUE_TEMPERATURE1_HIGH_LIMIT,
	W8378xx_VALUE_TEMPERATURE1_OVER_LIMIT = W8378xx_VALUE_TEMPERATURE1_HIGH_LIMIT,
	W8378xx_VALUE_TEMPERATURE1_LOW_LIMIT,
	W8378xx_VALUE_TEMPERATURE1_HYSTERESIS_LIMIT = W8378xx_VALUE_TEMPERATURE1_LOW_LIMIT,
	W8378xx_VALUE_FAN1_LIMIT,
	W8378xx_VALUE_FAN2_LIMIT,
	W8378xx_VALUE_FAN3_LIMIT
	};

#define W8378xx_VALUE_TEMPERATURE2       0x0150
#define W8378xx_VALUE_TEMPERATURE3       0x0250

#define	W8378xx_VALUE_TEMPERATURE2_HIGH_LIMIT			0x0155
#define	W8378xx_VALUE_TEMPERATURE2_OVER_LIMIT 			W8378xx_VALUE_TEMPERATURE2_HIGH_LIMIT
#define	W8378xx_VALUE_TEMPERATURE2_LOW_LIMIT			0x0153
#define	W8378xx_VALUE_TEMPERATURE2_HYSTERESIS_LIMIT 	W8378xx_VALUE_TEMPERATURE2_LOW_LIMIT

#define	W8378xx_VALUE_TEMPERATURE3_HIGH_LIMIT			0x0255
#define	W8378xx_VALUE_TEMPERATURE3_OVER_LIMIT 			W8378xx_VALUE_TEMPERATURE3_HIGH_LIMIT
#define	W8378xx_VALUE_TEMPERATURE3_LOW_LIMIT			0x0253
#define	W8378xx_VALUE_TEMPERATURE3_HYSTERESIS_LIMIT 	W8378xx_VALUE_TEMPERATURE3_LOW_LIMIT

#define W8378xx_VALUE_IN7      			0x0551
#define W8378xx_VALUE_IN8      			0x0550
#define	W8378xx_VALUE_IN7_HIGH_LIMIT	0x0500
#define	W8378xx_VALUE_IN7_LOW_LIMIT	 	0x0500
#define	W8378xx_VALUE_IN8_HIGH_LIMIT	0x0500
#define	W8378xx_VALUE_IN8_LOW_LIMIT		0x0500

// ---- Configuration Register (p19) Bank 0
#define	W8378xx_REG_CONFIG 				(0x040)
// Register flags / masks	
#define W8378xx_CONFIG_START 			0x01
#define W8378xx_CONFIG_EN_SMI 			0x02
#define W8378xx_CONFIG_EN_IRQ 			0x04
#define W8378xx_CONFIG_INT_CLEAR 		0x08
#define W8378xx_CONFIG_RST_OUT 			0x10
#define W8378xx_CONFIG_IRQPOL 			0x20
#define W8378xx_CONFIG_BEEP_GPO 		0x40
#define W8378xx_CONFIG_INITIALIZATION 	0x80

// ---- Interrupt Status Register 1 (p20) Bank 0				
// ---- SMI# Mask Register 1 (p21) Bank 0
// ---- IRQ Mask Register 1 (p22) Bank 0
#define W8378xx_REG_INTERRUPT_STATUS_1 	(0x041)
#define W8378xx_REG_SMI_MASK_1 			(0x043)				
#define W8378xx_REG_IRQ_MASK_1 			(0x045)				

// These last 3 registers flags / masks
#define W8378xx_INTERRUPT_IN0	(0x01)
#define W8378xx_INTERRUPT_IN1		(0x02)
#define W8378xx_INTERRUPT_IN2		(0x04)
#define W8378xx_INTERRUPT_IN3		(0x08)
#define W8378xx_INTERRUPT_TEMP1		(0x10)
#define W8378xx_INTERRUPT_TEMP23	(0x20)
#define W8378xx_INTERRUPT_FAN1		(0x40)
#define W8378xx_INTERRUPT_FAN2		(0x80)
// Differents names use by doc for the same *thing* in IRQ Mask Register 1	
#define W8378xx_INTERRUPT_V25INA	W8378xx_INTERRUPT_IN0
#define W8378xx_INTERRUPT_V25INB	W8378xx_INTERRUPT_IN1
#define W8378xx_INTERRUPT_V33IN		W8378xx_INTERRUPT_IN2
#define W8378xx_INTERRUPT_V5IN		W8378xx_INTERRUPT_IN3
#define W8378xx_INTERRUPT_VTHIN1	W8378xx_INTERRUPT_TEMP1
#define W8378xx_INTERRUPT_VTHIN23	W8378xx_INTERRUPT_TEMP23

// ---- Interrupt Status Register 2 (p21) Bank 0
// ---- SMI# Mask Register 2 (p22) Bank 0
// ---- IRQ Mask Register 2 (p23) Bank 0
#define W8378xx_REG_INTERRUPT_STATUS_2 	(0x042)		
#define W8378xx_REG_SMI_MASK_2 			(0x044)				
#define W8378xx_REG_IRQ_MASK_2 			(0x046)

// These last 3 registers flags / masks
#define W8378xx_INTERRUPT_IN4			(0x01)
#define W8378xx_INTERRUPT_MINUS_IN5		(0x02)
#define W8378xx_INTERRUPT_MINUS_IN6		(0x04)
#define W8378xx_INTERRUPT_FAN3			(0x08)
#define W8378xx_INTERRUPT_CASE_OPEN		(0x10)
						
// ---- VID/Fan Divisor Register (p23) Bank 0
#define W8378xx_REG_VID03_FAN12DIV 	(0x047)
	

enum {
	W8378xx_REG_SBUS_ADDRESS = 0x48,	// Serial Bus Address Register (p24)
	W8378xx_REG_VID4_ID,				// Voltage ID (p27)
	W8378xx_REG_TEMP23_SBUS_ADDRESS,	// Temperature 2 and Temperature 3 Serial Bus Address Register (p28)
	W8378xx_REG_FAN3DIV_PIN_CONTROL,	// Pin Control Register (p29)

	W8378xx_LAST
	};

// ---- Winbond Vendor ID register (p?) Bank 0
#define W8378xx_REG_WINBOND_VENDOR_ID 	(0x04f)

#define WINBOND_VENDOR_ID 	(0x5ca3)

// ---- Winbond Vendor ID register (p?) Bank 0
#define W8378xx_REG_WINBOND_VENDOR_ID 	(0x04f)

#define WINBOND_VENDOR_ID 	(0x5ca3)


// ---- Chip ID register (p?) Bank 0
#define W8378xx_REG_CHIP_ID 			(0x058)

#define W8378xx_CHIP_MASK	(0xFE)
#define W8378xx_CHIP_1D		(0x10)
#define W8378xx_CHIP_2D		(0x30)
#define W8378xx_CHIP_3S		(0x40)
#define W83781D		W8378xx_CHIP_1D
#define W83782D		W8378xx_CHIP_2D
#define W83783S		W8378xx_CHIP_3S

// ---- Bank Select & High Byte Access Register (p?) All Bank !!!
#define	W8378xx_REG_BANK_SELECT		(0x04e)

// ---- VBAT Monitor Control Register (p50) (w83782d / w83783s only)
#define W8378xx_REG_VBAT 	(0x5d)

// Usefull & useless (?!?) tools
// -----------------------------

#define W8378xx_FAN_UNDERRUN	1
#define W8378xx_FAN_OVERRUN		254
#define W8378xx_NO_FAN			(0xFF)
#define W8378xx_FAN_COUNT_TO_RPM( count, divisor )	\
	((count == 0) ? 0 : ((divisor == 0) ? 0 : ( 1350000 / ((count)*(divisor)) ) ) )


// Some constantes
// ---------------

// Real voltage is reduced to range 0,4096 mV, if needed
// Positive voltage : documentation say : in = v * r2 / (r1 + r2)
// so, v = in * (r1 + r2) / r2

// For 12V, Winbond recommand R1 = 28k Ohms, R2 = 10k Ohms 
#define W8378xx_12V_MUL		(28 + 10)	// (R1 + R2)
#define W8378xx_12V_DIV		10			// R2

// For 5V, Winbond recommand R1 = 34k Ohms, R2 = 50k Ohms 
#define W8378xx_5V_MUL		(34 + 50)	// (R1 + R2)
#define W8378xx_5V_DIV		50			// R2
				
// Negative voltage : documentation say : in4 = v * r2 / (r1 + r2)
// so, v = in4 * (r1 + r2) / r2
// For -12V, Winbond recommand R1 = 232k Ohms, R2 = 56k Ohms
#define W8378xx_VREF			(3600 / 16) // 3.6V, expressed in 16mV unit !
#define W8378xx_N12V_B_FACTOR	(232 / (232 + 56))	// (R1 + R2)

// For -5V, Winbond recommand R1 = 34k Ohms, R2 = 50k Ohms 
#define W8378xx_N5V_MUL		(34 + 50)	// (R1 + R2)
#define W8378xx_N5V_DIV		50			// R2


// Definitions lists
// -----------------

// Voltage ID table, unit is mV, 0 mean *not supported* !
int g_w8378xx_VID[] = { 2050, 2000, 1950, 1900, 1850, 1800, 0, 0, 
						0, 0, 0, 0, 0, 0, 0, 0,
						3500, 3400, 3300, 3200, 3100, 3000, 2900, 2800,
						2700, 2600, 2500, 2400, 2300, 2200, 2100, 0}; 
						


// Supported w8378xx infos
typedef struct {
	char *	name;
	int		chip_id;
	long	mask;		// mask for supported sensors, see in g_w8378xx_sensors
	int		io_base;
	int		current_reg_bank;
} W8378xx;

W8378xx g_w8378xx_supported[] = {
	{ "w83781d", W83781D, 1, W8378xx_IO_BASE, 0 },
	{ "w83782d", W83782D, 2, W8378xx_IO_BASE, 0 },
	{ "w83783s", W83783S, 4, W8378xx_IO_BASE, 0 },
	{ NULL } // NULL name == end of *list*
};

// Supported sensors infos
typedef struct 
	{
	char *		name;
	int64		unsupported;	// bit 0 set if unsupported by first chip in g_w8378xx_supported, etc
	SENSOR_KIND	kind;
	long		flags;
	int			value_multiplier;
	int			value_divider;
	}
W8378xx_SENSORS;


W8378xx_SENSORS g_w8378xx_sensors[] = {
	{ "in0", 		0,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, 16, 1},
	{ "in1", 		4,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, 16, 1},
	{ "in2", 		0,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, 16, 1},
	{ "in3", 		0,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, 16 * W8378xx_5V_MUL, W8378xx_5V_DIV},
	{ "in4", 		0,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, 16 * W8378xx_12V_MUL, W8378xx_12V_DIV},
	{ "in5", 		0,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, -7000, 125},
	{ "in6", 		0,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, -3000, 125},
	{ "in7", 		1,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, 16, 1},
	{ "in8", 		1,		SENSOR_KIND_VOLTAGE, 		SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LIMITS, 16, 1},
	{ "temp1", 		0,		SENSOR_KIND_TEMPERATURE, 	SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_HIGH_LIMIT, 1, 10},
	{ "temp2", 		0,		SENSOR_KIND_TEMPERATURE, 	SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_HIGH_LIMIT, 1, 10},
	{ "temp3", 		4,		SENSOR_KIND_TEMPERATURE, 	SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_HIGH_LIMIT, 1, 10},
	{ "fan1", 		0,		SENSOR_KIND_FAN, 			SENSOR_FLAG_CONTROLABLE | SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LOW_LIMIT, 1, 1 },
	{ "fan2", 		0,		SENSOR_KIND_FAN, 			SENSOR_FLAG_CONTROLABLE | SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LOW_LIMIT, 1, 1 },
	{ "fan3", 		0,		SENSOR_KIND_FAN, 			SENSOR_FLAG_CONTROLABLE | SENSOR_FLAG_ALARMABLE | SENSOR_FLAG_LOW_LIMIT, 1, 1 },
	{ "chassis", 	0,		SENSOR_KIND_STATE, 			0 },
	{ "vid",		0,		SENSOR_KIND_VOLTAGE, 		0, 1, 1 },
	{ NULL }	// NULL name == end of *list*
};



#endif	// #ifndef W8378XX_H
